@@ -1,6 +1,10 @@
 import logging
-from typing import Dict, Any, Optional
 from collections import defaultdict
+from typing import Dict, Any, Optional
+
+import click
+
+from .click_common import command, format_output
 from .device import Device
 from .utils import deprecated
 
@@ -84,6 +88,9 @@ class ChuangmiPlugStatus:
              self.wifi_led)
         return s
 
+    def __json__(self):
+        return self.data
+
 
 class ChuangmiPlug(Device):
     """Main class representing the Chuangmi Plug V1 and V3."""
@@ -98,9 +105,18 @@ class ChuangmiPlug(Device):
         else:
             self.model = MODEL_CHUANGMI_PLUG_M1
 
+    @command(
+        default_output=format_output(
+            "",
+            "Power: {result.power}\n"
+            "USB Power: {result.usb_power}\n"
+            "Temperature: {result.temperature} °C\n"
+            "Load power: {result.load_power}\n"
+            "WiFi LED: {result.wifi_led}\n")
+    )
     def status(self) -> ChuangmiPlugStatus:
         """Retrieve properties."""
-        properties = AVAILABLE_PROPERTIES[self.model]
+        properties = AVAILABLE_PROPERTIES[self.model].copy()
         values = self.send(
             "get_prop",
             properties
@@ -123,6 +139,9 @@ class ChuangmiPlug(Device):
         return ChuangmiPlugStatus(
             defaultdict(lambda: None, zip(properties, values)))
 
+    @command(
+        default_output = format_output("Powering on"),
+    )
     def on(self):
         """Power on."""
         if self.model == MODEL_CHUANGMI_PLUG_V1:
@@ -130,6 +149,9 @@ class ChuangmiPlug(Device):
 
         return self.send("set_power", ["on"])
 
+    @command(
+        default_output = format_output("Powering off"),
+    )
     def off(self):
         """Power off."""
         if self.model == MODEL_CHUANGMI_PLUG_V1:
@@ -137,14 +159,27 @@ class ChuangmiPlug(Device):
 
         return self.send("set_power", ["off"])
 
+    @command(
+        default_output = format_output("Powering USB on"),
+    )
     def usb_on(self):
         """Power on."""
         return self.send("set_usb_on", [])
 
+    @command(
+        default_output = format_output("Powering USB off"),
+    )
     def usb_off(self):
         """Power off."""
         return self.send("set_usb_off", [])
 
+    @command(
+        click.argument("wifi_led", type=bool),
+        default_output=format_output(
+            lambda wifi_led: "Turning on WiFi LED"
+            if wifi_led else "Turning off WiFi LED"
+        )
+    )
     def set_wifi_led(self, led: bool):
         """Set the wifi led on/off."""
         if led:
