@@ -1,18 +1,24 @@
 # -*- coding: UTF-8 -*-
-import logging
-import click
-import pretty_cron
 import ast
-import sys
 import json
-import time
+import logging
 import pathlib
-from appdirs import user_cache_dir
+import sys
+import threading
+import time
 from pprint import pformat as pf
 from typing import Any  # noqa: F401
-from miio.click_common import (ExceptionHandlerGroup, validate_ip,
-                               validate_token)
+
+import click
+import pretty_cron
+from appdirs import user_cache_dir
+from tqdm import tqdm
+
 import miio  # noqa: E402
+from miio.click_common import (ExceptionHandlerGroup, validate_ip,
+                               validate_token, )
+from .device import UpdateState
+from .updater import OneShotServer
 
 _LOGGER = logging.getLogger(__name__)
 pass_dev = click.make_pass_decorator(miio.Device, ensure=True)
@@ -425,26 +431,56 @@ def sound(vac: miio.Vacuum, volume: int, test_mode: bool):
 
 @cli.command()
 @click.argument('url')
-@click.argument('md5sum')
-@click.argument('sid', type=int)
+@click.argument('md5sum', required=False, default=None)
+@click.option('--sid', type=int, required=False, default=10000)
+@click.option('--ip', required=False)
 @pass_dev
-def install_sound(vac: miio.Vacuum, url: str, md5sum: str, sid: int):
-    """Install a sound."""
+def install_sound(vac: miio.Vacuum, url: str, md5sum: str, sid: int, ip: str):
+    """Install a sound.
+
+    When passing a local file this will create a self-hosting server
+     for the given file and the md5sum will be calculated automatically.
+
+    For URLs you have to specify the md5sum manually.
+
+    `--ip` can be used to override automatically detected IP address for
+     the device to contact for the update.
+    """
     click.echo("Installing from %s (md5: %s) for id %s" % (url, md5sum, sid))
-    click.echo(vac.install_sound(url, md5sum, sid))
+
+    local_url = None
+    server = None
+    if url.startswith("http"):
+        if md5sum is None:
+            click.echo("You need to pass md5 when using URL for updating.")
+            return
+        local_url = url
+    else:
+        server = OneShotServer(url)
+        local_url = server.url(ip)
+        md5sum = server.md5
+
+        t = threading.Thread(target=server.serve_once)
+        t.start()
+        click.echo("Hosting file at %s" % local_url)
+
+    click.echo(vac.install_sound(local_url, md5sum, sid))
 
     progress = vac.sound_install_progress()
     while progress.is_installing:
-        print(progress)
         progress = vac.sound_install_progress()
-        time.sleep(0.1)
+        print("%s (%s %%)" % (progress.state.name, progress.progress))
+        time.sleep(1)
 
     progress = vac.sound_install_progress()
 
-    if progress.progress == 100 and progress.error == 0:
-        click.echo("Installation of sid '%s' complete!" % progress.sid)
-    else:
+    if progress.is_errored:
         click.echo("Error during installation: %s" % progress.error)
+    else:
+        click.echo("Installation of sid '%s' complete!" % sid)
+
+    if server is not None:
+        t.join()
 
 @cli.command()
 @pass_dev
@@ -465,6 +501,16 @@ def timezone(vac: miio.Vacuum, tz=None):
         click.echo("Timezone: %s" % vac.timezone())
 
 
+@cli.command()
+@click.argument('enabled', required=False, type=bool)
+@pass_dev
+def carpet_mode(vac: miio.Vacuum, enabled=None):
+    """Query or set the carpet mode."""
+    if enabled is None:
+        click.echo(vac.carpet_mode())
+    else:
+        click.echo(vac.set_carpet_mode(enabled))
+
 @cli.command()
 @click.argument('ssid', required=True)
 @click.argument('password', required=True)
@@ -481,6 +527,74 @@ def configure_wifi(vac: miio.Vacuum, ssid: str, password: str,
     click.echo(vac.configure_wifi(ssid, password, uid, timezone))
 
 
+@cli.command()
+@pass_dev
+def update_status(vac: miio.Vacuum):
+    """Return update state and progress."""
+    update_state = vac.update_state()
+    click.echo("Update state: %s" % update_state)
+
+    if update_state == UpdateState.Downloading:
+        click.echo("Update progress: %s" % vac.update_progress())
+
+
+@cli.command()
+@click.argument('url', required=True)
+@click.argument('md5', required=False, default=None)
+@click.option('--ip', required=False)
+@pass_dev
+def update_firmware(vac: miio.Vacuum, url: str, md5: str, ip: str):
+    """Update device firmware.
+
+    If `url` starts with http* it is expected to be an URL.
+     In that case md5sum of the file has to be given.
+
+    `--ip` can be used to override automatically detected IP address for
+     the device to contact for the update.
+     """
+
+    # TODO Check that the device is in updateable state.
+
+    click.echo("Going to update from %s" % url)
+    if url.lower().startswith("http"):
+        if md5 is None:
+            click.echo("You need to pass md5 when using URL for updating.")
+            return
+
+        click.echo("Using %s (md5: %s)" % (url, md5))
+    else:
+        server = OneShotServer(url)
+        url = server.url(ip)
+
+        t = threading.Thread(target=server.serve_once)
+        t.start()
+        click.echo("Hosting file at %s" % url)
+        md5 = server.md5
+
+    update_res = vac.update(url, md5)
+    if update_res:
+        click.echo("Update started!")
+    else:
+        click.echo("Starting the update failed: %s" % update_res)
+
+    with tqdm(total=100) as t:
+        state = vac.update_state()
+        while state == UpdateState.Downloading:
+            try:
+                state = vac.update_state()
+                progress = vac.update_progress()
+            except:  # we may not get our messages through during upload
+                continue
+
+            if state == UpdateState.Installing:
+                click.echo("Installation started, please wait until the vacuum reboots")
+                break
+
+            t.update(progress - t.n)
+            t.set_description("%s" % state.name)
+            time.sleep(1)
+
+
 @cli.command()
 @click.argument('cmd', required=True)
 @click.argument('parameters', required=False)
